<?php 
/**
 * Plugin Name: Site Access Provisioner (Matomo Plugin)
 * Plugin URI: http://plugins.matomo.org/SiteAccessProvisioner
 * Description: Provides a simple method to provision access to site analytics. Integration with Content Management System is highly advised.
 * Author: Josh Brule
 * Author URI: https://github.com/jbrule/matomoplugin-SiteAccessProvisioner
 * Version: 3.0.1
 */
?><?php
/**
 * Piwik - free/libre analytics platform
 *
 * @link http://piwik.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html GPL v3 or later
 */

namespace Piwik\Plugins\SiteAccessProvisioner;

use Piwik\Piwik;
use Piwik\Access;
use Piwik\Common;
use Piwik\Option;
use Piwik\Settings\Storage;
use Piwik\Settings\Setting;
use Piwik\Plugins\SitesManager\API as APISitesManager;
use Piwik\Plugins\UsersManager\API as APIUsersManager;

 
if (defined( 'ABSPATH')
&& function_exists('add_action')) {
    $path = '/matomo/app/core/Plugin.php';
    if (defined('WP_PLUGIN_DIR') && WP_PLUGIN_DIR && file_exists(WP_PLUGIN_DIR . $path)) {
        require_once WP_PLUGIN_DIR . $path;
    } elseif (defined('WPMU_PLUGIN_DIR') && WPMU_PLUGIN_DIR && file_exists(WPMU_PLUGIN_DIR . $path)) {
        require_once WPMU_PLUGIN_DIR . $path;
    } else {
        return;
    }
    add_action('plugins_loaded', function () {
        if (function_exists('matomo_add_plugin')) {
            matomo_add_plugin(__DIR__, __FILE__, true);
        }
    });
}

class SiteAccessProvisioner extends \Piwik\Plugin
{
	public static function processRequest($idUser, $site, $timestamp, $token)
	{
		$settings = new SystemSettings();
		
		$sharedSecret = $settings->sharedSecret->getValue();
		
		if(empty($sharedSecret))
		{
			throw new \Exception("Plugin function is inactive. Shared secret not set.");
		}
		
		$tokenttl = $settings->tokenttl->getValue();
		
		$enforceMatchingUserId = $settings->enforceMatchingUserId->getValue();
		
		$msg = array();
		
		$idSite = false;
		
		$computedToken = hash('sha256',implode('',[$sharedSecret, $idUser, $site, $timestamp]));
		$currentTimestamp = time();
		
		if($token === $computedToken && (!$enforceMatchingUserId || ($enforceMatchingUserId && strtolower($idUser) == strtolower(Piwik::getCurrentUserLogin()))))
		{
			if(($currentTimestamp - $timestamp) <= $tokenttl)
			{
				$idSite = (is_int($site))? $site : self::getIdSiteFromUrl($site);
				
				if($idSite !== false)
				{
					if(!Piwik::isUserHasViewAccess($idSite) && !Piwik::isUserHasAdminAccess($idSite) && !Piwik::hasUserSuperUserAccess())
					{
						self::grantSiteAccess($idSite,"view");
						if(Piwik::isUserHasViewAccess($idSite))
						{
							$msg["status"] = "success";
							$msg["message"] = sprintf("You were successfully granted view access");
						}
					}
				}else{
					$msg["status"] = "error";
					$msg["message"] = "Could not get idSite";
				}
				
			}else{
				$msg["status"] = "error";
				$msg["message"] = "Access token has expired";
			}
		}else{
			$msg["status"] = "error";
			$msg["message"] = "Invalid token";
		}
		
		return [$msg, $idSite];
	}
	
	private static function getIdSiteFromUrl($url)
	{
		$result = Access::doAsSuperUser(function() use ($url){
			$sitesManager = APISitesManager::getInstance();
			return $sitesManager->getSitesIdFromSiteUrl($url);
		});
		
		return (!empty($result))? $result[0]["idsite"] : false;
	}
	
	private static function grantSiteAccess($idSite, $accessLevel)
	{
		$login = Piwik::getCurrentUserLogin();
		
		return Access::doAsSuperUser(function() use ($login, $accessLevel, $idSite){
			$usersManager = APIUsersManager::getInstance();
			return $usersManager->setUserAccess($login, $accessLevel, $idSite);
		});
	}
}
